@@ -1,6 +1,6 @@
-/* $Id: rmagick.h,v 1.82 2005/04/28 23:41:54 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.110 2006/06/02 23:26:53 rmagick Exp $ */
 /*=============================================================================
-|               Copyright (C) 2005 by Timothy P. Hunter
+|               Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmagick.h
 | Purpose:  RMagick declarations and definitions
 | Author:   Tim Hunter
@@ -19,6 +19,14 @@
 #include "ruby.h"
 #include "intern.h"
 #include "rubyio.h"
+
+
+// Undef Ruby's versions of these symbols
+#undef PACKAGE_VERSION
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_TARNAME
 #include "magick/api.h"
 
 
@@ -43,10 +51,15 @@
 #define RUBY16(d) d
 #endif
 
+// Backport these definitions to Ruby 1.6.7
 #if !defined(ULONG2NUM)
 #define ULONG2NUM(v) UINT2NUM(v)
 #endif
 
+#if !defined(LONG2NUM)
+#define LONG2NUM(v) INT2NUM(v)
+#endif
+
 
 // Define a version of StringValuePtr that works in both 1.6 and 1.8.
 #if !defined(StringValuePtr)
@@ -61,6 +74,10 @@
 #undef DegreesToRadians     // defined in ImageMagick.h in 6.0.2
 #define DegreesToRadians(x) ((x)*3.14159265358979323846/180.0)
 
+#define LMAX(a,b) ((((long)(a))>((long)(b)))?((long)(a)):((long)(b)))
+#define FMAX(a,b) ((((double)(a))>((double)(b)))?((double)(a)):((double)(b)))
+#define FMIN(a,b) ((((double)(a))<=((double)(b)))?((double)(a)):((double)(b)))
+
 typedef ImageInfo Info; // Make type name match class name
 typedef PixelPacket Pixel;
 
@@ -98,8 +115,7 @@ typedef enum
 {
     False = 0,
     True = 1
-}
-boolean;
+} boolean;
 
 typedef enum {
     AnyWeight,
@@ -160,34 +176,40 @@ typedef enum _QuantumExpressionOperator
     ImageMagick used simply size_t and off_t in 5.5.1, then defined the
     Extended(Un)SignedIntegralType from 5.5.2 thru 5.5.7. The 5.5.8 release
     deprecates these types and uses Magick(Un)SignedType instead.
-    GraphicsMagick 1.1. introduced the magick_(u)int64_t type.
+    Prior to 1.1, GraphicsMagick defined the Extended(Un)SignedIntegralType(s).
+    GraphicsMagick 1.1. introduced the magick_(u)int64_t types.
 
-    Here, if we don't already have magick_(u)int64_t, define them.
+    Here, if we don't already have magick_(u)int64_t, define them in terms
+    of whatever other types are defined.
 */
 #if !defined(HAVE_MAGICK_INT64_T)
-#if defined(HAVE_MAGICKOFFSETTYPE)
-typedef MagickOffsetType magick_int64_t;
-#elif defined(HAVE_EXTENDEDSIGNEDINTEGRALTYPE)
-typedef ExtendedSignedIntegralType magick_int64_t;
-#else
-typedef off_t magick_int64_t;
-#endif
+    #if defined(HAVE_MAGICKOFFSETTYPE)
+        typedef MagickOffsetType magick_int64_t;
+    #elif defined(HAVE_EXTENDEDSIGNEDINTEGRALTYPE)
+        typedef ExtendedSignedIntegralType magick_int64_t;
+    #else
+        typedef off_t magick_int64_t;
+    #endif
 #endif
 
 #if !defined(HAVE_MAGICK_UINT64_T)
-#if defined(HAVE_MAGICKSIZETYPE)
-typedef MagickSizeType magick_uint64_t;
-#elif defined(HAVE_EXTENDEDUNSIGNEDINTEGRALTYPE)
-typedef ExtendedUnsignedIntegralType magick_uint64_t;
-#else
-typedef size_t magick_uint64_t;
-#endif
+    #if defined(HAVE_MAGICKSIZETYPE)
+        typedef MagickSizeType magick_uint64_t;
+    #elif defined(HAVE_EXTENDEDUNSIGNEDINTEGRALTYPE)
+        typedef ExtendedUnsignedIntegralType magick_uint64_t;
+    #else
+        typedef size_t magick_uint64_t;
+    #endif
 #endif
 
 #if !defined(HAVE_MAGICKBOOLEANTYPE)
 typedef unsigned int MagickBooleanType;
 #endif
 
+#if !defined(HAVE_UNDEFINEDGRAVITY)
+#define UndefinedGravity 0
+#endif
+
 // This implements the "omitted storage class model" for external variables.
 // (Ref: Harbison & Steele.) The rmmain.c file defines MAIN, which causes
 // the single defining declarations to be generated. No other source files
@@ -251,24 +273,29 @@ EXTERN VALUE Class_FilterTypes;
 EXTERN VALUE Class_GravityType;
 EXTERN VALUE Class_ImageType;
 EXTERN VALUE Class_InterlaceType;
+#if defined(HAVE_COMPAREIMAGELAYERS)
+EXTERN VALUE Class_MagickLayerMethod;
+#endif
 EXTERN VALUE Class_NoiseType;
+#if defined(HAVE_IMAGE_ORIENTATION)
+EXTERN VALUE Class_OrientationType;
+#endif
 EXTERN VALUE Class_PaintMethod;
 EXTERN VALUE Class_PreviewType;
 EXTERN VALUE Class_RenderingIntent;
 EXTERN VALUE Class_ResolutionType;
+EXTERN VALUE Class_StorageType;
 EXTERN VALUE Class_StretchType;
 EXTERN VALUE Class_StyleType;
 EXTERN VALUE Class_WeightType;
-#if defined(HAVE_IMAGE_ORIENTATION)
-EXTERN VALUE Class_OrientationType;
-#endif
+EXTERN VALUE Class_VirtualPixelMethod;
 
 /*
 *   Commonly-used IDs
 */
 EXTERN ID ID__dummy_img_;       // "_dummy_img_"
-EXTERN ID ID_changed;           // "changed"
 EXTERN ID ID_call;              // "call"
+EXTERN ID ID_changed;           // "changed"
 EXTERN ID ID_cur_image;         // "cur_image"
 EXTERN ID ID_dup;               // "dup"
 EXTERN ID ID_enumerators;       // "enumerators"
@@ -291,19 +318,6 @@ EXTERN ID ID_x;                 // "x"
 EXTERN ID ID_y;                 // "y"
 
 
-#if defined(HAVE_GETNEXTIMAGEINLIST)
-#define GET_NEXT_IMAGE(a) GetNextImageInList(a)
-#else
-#define GET_NEXT_IMAGE(a) (a)->next
-#endif
-
-#if defined(HAVE_GETLOCALEEXCEPTIONMESSAGE)
-#define GET_MSG(s,t) GetLocaleExceptionMessage((s), (t))
-#else
-#define GET_MSG(s,t) t
-#endif
-
-
 #define min(a,b) ((a)<(b)?(a):(b))
 #define max(a,b) ((a)>(b)?(a):(b))
 
@@ -312,14 +326,8 @@ EXTERN ID ID_y;                 // "y"
 
 /*
    Handle warnings & errors
-
-   For ExceptionInfo structures in auto storage. Destroys the
-   ExceptionInfo structure (releasing any IM storage) before
-   calling rb_raise.
 */
-#define HANDLE_ERROR rm_handle_error(&exception);
-// For ExceptionInfo structures in images.
-#define HANDLE_ERROR_IMG(img) rm_handle_error(&((img)->exception));
+#define CHECK_EXCEPTION() rm_check_exception(&exception, NULL, RetainOnError);
 
 /*
     Map the QuantumDepth to a StorageType.
@@ -399,6 +407,25 @@ EXTERN ID ID_y;                 // "y"
     ATTR_READER(class, attr)\
     ATTR_WRITER(class, attr)
 
+/*
+ *  Define functions to get/set attributes in Image::Info that
+ *  use the Get/SetImageOption API.
+*/
+#define OPTION_ATTR_READER(opt, key) \
+    VALUE Info_##opt(VALUE self)\
+    {\
+        return get_option(self, #key);\
+    }
+#define OPTION_ATTR_WRITER(opt, key) \
+    VALUE Info_##opt##_eq(VALUE self, VALUE string)\
+    {\
+        return set_option(self, #key, string);\
+    }
+#define OPTION_ATTR_ACCESSOR(opt, key)\
+    OPTION_ATTR_READER(opt, key)\
+    OPTION_ATTR_WRITER(opt, key)
+
+
 /*
  *  Declare Pixel channel attribute writers
 */
@@ -417,6 +444,33 @@ Pixel_##_channel_##_eq(VALUE self, VALUE v) \
 }
 
 
+/*
+ *  Declare Pixel CMYK channel attribute accessors
+*/
+#define DEF_PIXEL_CMYK_CHANNEL_ACCESSOR(_cmyk_channel_, _rgb_channel_) \
+extern VALUE \
+Pixel_##_cmyk_channel_##_eq(VALUE self, VALUE v) \
+{ \
+    Pixel *pixel; \
+ \
+    rm_check_frozen(self); \
+    Data_Get_Struct(self, Pixel, pixel); \
+    pixel->_rgb_channel_ = (Quantum) NUM2UINT(v); \
+    rb_funcall(self, ID_changed, 0); \
+    rb_funcall(self, ID_notify_observers, 1, self); \
+    return INT2NUM(pixel->_rgb_channel_); \
+} \
+ \
+extern VALUE \
+Pixel_##_cmyk_channel_(VALUE self) \
+{ \
+    Pixel *pixel; \
+ \
+    Data_Get_Struct(self, Pixel, pixel); \
+    return INT2NUM(pixel->_rgb_channel_); \
+}
+
+
 /*
  *  Enum constants - define a subclass of Enum for the specified enumeration.
  *  Define an instance of the subclass for each member in the enumeration.
@@ -446,6 +500,7 @@ Pixel_##_channel_##_eq(VALUE self, VALUE v) \
    e = (type)(magick_enum->val);\
    } while(0)
 
+#define ENUM_TO_NAME(_enum) case _enum: return #_enum;
 
 
 // Method, external function declarations. These declarations are
@@ -458,6 +513,7 @@ Pixel_##_channel_##_eq(VALUE self, VALUE v) \
 // rmdraw.c
 ATTR_WRITER(Draw, affine)
 ATTR_WRITER(Draw, align)
+ATTR_WRITER(Draw, border_color)
 ATTR_WRITER(Draw, decorate)
 ATTR_WRITER(Draw, density)
 ATTR_WRITER(Draw, encoding)
@@ -533,6 +589,7 @@ extern VALUE ImageList_map(VALUE, VALUE, VALUE);
 extern VALUE ImageList_montage(VALUE);
 extern VALUE ImageList_morph(VALUE, VALUE);
 extern VALUE ImageList_mosaic(VALUE);
+extern VALUE ImageList_optimize_layers(VALUE, VALUE);
 extern VALUE ImageList_quantize(int, VALUE*, VALUE);
 extern VALUE ImageList_to_blob(VALUE);
 extern VALUE ImageList_write(VALUE, VALUE);
@@ -547,29 +604,39 @@ extern void rm_imagelist_push(VALUE, VALUE);
 
 // rminfo.c
 ATTR_ACCESSOR(Info, antialias)
+ATTR_ACCESSOR(Info, authenticate)
 ATTR_ACCESSOR(Info, background_color)
 ATTR_ACCESSOR(Info, border_color)
 ATTR_ACCESSOR(Info, colorspace)
+ATTR_ACCESSOR(Info, comment)
 ATTR_ACCESSOR(Info, compression)
+ATTR_ACCESSOR(Info, delay)
 ATTR_ACCESSOR(Info, density)
 ATTR_ACCESSOR(Info, depth)
+ATTR_ACCESSOR(Info, dispose)
 ATTR_ACCESSOR(Info, dither)
 ATTR_ACCESSOR(Info, extract)
 ATTR_ACCESSOR(Info, filename)
+ATTR_ACCESSOR(Info, fill)
 ATTR_ACCESSOR(Info, font)
 ATTR_ACCESSOR(Info, format)
 ATTR_ACCESSOR(Info, fuzz)
+ATTR_ACCESSOR(Info, gravity)
 ATTR_ACCESSOR(Info, group)
 ATTR_ACCESSOR(Info, image_type)
 ATTR_ACCESSOR(Info, interlace)
+ATTR_ACCESSOR(Info, label)
 ATTR_ACCESSOR(Info, matte_color)
+ATTR_WRITER(Info, monitor)
 ATTR_ACCESSOR(Info, monochrome)
 ATTR_ACCESSOR(Info, number_scenes)
+ATTR_ACCESSOR(Info, orientation)
 ATTR_ACCESSOR(Info, page)
 ATTR_ACCESSOR(Info, pen)
 // ATTR_ACCESSOR(Info, ping) obsolete
-// ATTR_ACCESSOR(Info, pointsize)
+ATTR_ACCESSOR(Info, pointsize)
 ATTR_ACCESSOR(Info, quality)
+ATTR_ACCESSOR(Info, sampling_factor)
 ATTR_ACCESSOR(Info, scene)
 ATTR_ACCESSOR(Info, server_name)
 ATTR_ACCESSOR(Info, subimage)
@@ -603,6 +670,7 @@ ATTR_ACCESSOR(Image, blur)
 ATTR_ACCESSOR(Image, border_color)
 ATTR_READER(Image, bounding_box)
 ATTR_ACCESSOR(Image, chromaticity)
+ATTR_ACCESSOR(Image, class_type)
 ATTR_WRITER(Image, clip_mask)
 ATTR_ACCESSOR(Image, color_profile)
 ATTR_READER(Image, colors)
@@ -632,6 +700,7 @@ ATTR_ACCESSOR(Image, matte)
 ATTR_ACCESSOR(Image, matte_color)
 ATTR_READER(Image, mean_error_per_pixel)
 ATTR_READER(Image, mime_type)
+ATTR_WRITER(Image, monitor)
 ATTR_ACCESSOR(Image, montage)
 ATTR_READER(Image, normalized_mean_error)
 ATTR_READER(Image, normalized_maximum_error)
@@ -646,10 +715,11 @@ ATTR_ACCESSOR(Image, rendering_intent)
 ATTR_READER(Image, rows)
 ATTR_READER(Image, scene)
 ATTR_ACCESSOR(Image, start_loop)
-ATTR_ACCESSOR(Image, class_type)
+ATTR_ACCESSOR(Image, ticks_per_second)
 ATTR_ACCESSOR(Image, tile_info)
 ATTR_READER(Image, total_colors)
 ATTR_ACCESSOR(Image, units)
+ATTR_ACCESSOR(Image, virtual_pixel_method)
 ATTR_ACCESSOR(Image, x_resolution)
 ATTR_ACCESSOR(Image, y_resolution)
 
@@ -663,6 +733,7 @@ extern VALUE Image_initialize(VALUE, VALUE, VALUE, VALUE, VALUE);
 
 extern VALUE Image_adaptive_threshold(int, VALUE *, VALUE);
 extern VALUE Image_add_noise(VALUE, VALUE);
+extern VALUE Image_add_noise_channel(int, VALUE *, VALUE);
 extern VALUE Image_affine_transform(VALUE, VALUE);
 extern VALUE Image_aref(VALUE, VALUE);
 extern VALUE Image_aset(VALUE, VALUE, VALUE);
@@ -695,6 +766,7 @@ extern VALUE Image_composite_affine(VALUE, VALUE, VALUE);
 extern VALUE Image_compress_colormap_bang(VALUE);
 extern VALUE Image_constitute(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_contrast(int, VALUE *, VALUE);
+extern VALUE Image_contrast_stretch_channel(int, VALUE *, VALUE);
 extern VALUE Image_convolve(VALUE, VALUE, VALUE);
 extern VALUE Image_convolve_channel(int, VALUE *, VALUE);
 extern VALUE Image_copy(VALUE);
@@ -705,6 +777,7 @@ extern VALUE Image_despeckle(VALUE);
 extern VALUE Image_difference(VALUE, VALUE);
 extern VALUE Image_dispatch(int, VALUE *, VALUE);
 extern VALUE Image_display(VALUE);
+extern VALUE Image_distortion_channel(int, VALUE *, VALUE);
 extern VALUE Image__dump(VALUE, VALUE);
 extern VALUE Image_dup(VALUE);
 extern VALUE Image_each_profile(VALUE);
@@ -713,7 +786,8 @@ extern VALUE Image_emboss(int, VALUE *, VALUE);
 extern VALUE Image_enhance(VALUE);
 extern VALUE Image_equalize(VALUE);
 extern VALUE Image_erase_bang(VALUE);
-extern VALUE Image_export_pixels(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_export_pixels(int, VALUE *, VALUE);
+extern VALUE Image_export_pixels_to_str(int, VALUE *, VALUE);
 extern VALUE Image_flip(VALUE);
 extern VALUE Image_flip_bang(VALUE);
 extern VALUE Image_flop(VALUE);
@@ -728,10 +802,10 @@ extern VALUE Image_get_pixels(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_gray_q(VALUE);
 extern VALUE Image_grayscale_pseudo_class(int, VALUE *, VALUE);
 extern VALUE Image_implode(int, VALUE *, VALUE);
-extern VALUE Image_import_pixels(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_import_pixels(int, VALUE *, VALUE);
 extern VALUE Image_init_copy(VALUE, VALUE);
 extern VALUE Image_inspect(VALUE);
-extern VALUE Image_level(int, VALUE *, VALUE);
+extern VALUE Image_level2(int, VALUE *, VALUE);
 extern VALUE Image_level_channel(int, VALUE *, VALUE);
 extern VALUE Image__load(VALUE, VALUE);
 extern VALUE Image_magnify(VALUE);
@@ -751,7 +825,7 @@ extern VALUE Image_normalize_channel(int, VALUE *, VALUE);
 extern VALUE Image_oil_paint(int, VALUE *, VALUE);
 extern VALUE Image_opaque(VALUE, VALUE, VALUE);
 extern VALUE Image_opaque_q(VALUE);
-extern VALUE Image_ordered_dither(VALUE);
+extern VALUE Image_ordered_dither(int, VALUE *, VALUE);
 extern VALUE Image_palette_q(VALUE);
 extern VALUE Image_ping(VALUE, VALUE);
 extern VALUE Image_pixel_color(int, VALUE *, VALUE);
@@ -812,6 +886,7 @@ extern VALUE Image_trim(VALUE);
 extern VALUE Image_trim_bang(VALUE);
 extern VALUE Image_unsharp_mask(int, VALUE *, VALUE);
 extern VALUE Image_unsharp_mask_channel(int, VALUE *, VALUE);
+extern VALUE Image_vignette(int, VALUE *, VALUE);
 extern VALUE Image_wave(int, VALUE *, VALUE);
 extern VALUE Image_white_threshold(int, VALUE *, VALUE);
 extern VALUE Image_write(VALUE, VALUE);
@@ -847,7 +922,9 @@ extern VALUE  Color_to_s(VALUE);
 extern VALUE  Color_from_ColorInfo(const ColorInfo *);
 extern VALUE  ClassType_new(ClassType);
 extern VALUE  ColorspaceType_new(ColorspaceType);
+extern VALUE  CompositeOperator_new(CompositeOperator);
 extern VALUE  CompressionType_new(CompressionType);
+extern VALUE  DisposeType_new(DisposeType);
 extern VALUE  EndianType_new(EndianType);
 extern VALUE  FilterTypes_new(FilterTypes);
 extern VALUE  Font_to_s(VALUE);
@@ -865,6 +942,10 @@ ATTR_ACCESSOR(Pixel, red)
 ATTR_ACCESSOR(Pixel, green)
 ATTR_ACCESSOR(Pixel, blue)
 ATTR_ACCESSOR(Pixel, opacity)
+ATTR_ACCESSOR(Pixel, cyan)
+ATTR_ACCESSOR(Pixel, magenta)
+ATTR_ACCESSOR(Pixel, yellow)
+ATTR_ACCESSOR(Pixel, black)
 extern VALUE  Pixel_case_eq(VALUE, VALUE);
 extern VALUE  Pixel_clone(VALUE);
 extern VALUE  Pixel_dup(VALUE);
@@ -909,6 +990,8 @@ extern VALUE  Statistics_new(ImageStatistics *);
 #if defined(HAVE_IMAGE_ORIENTATION)
 extern VALUE   OrientationType_new(OrientationType);
 #endif
+extern const char *StorageType_name(StorageType);
+extern VALUE   VirtualPixelMethod_new(VirtualPixelMethod);
 
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 extern VALUE  Enum_alloc(VALUE);
@@ -934,19 +1017,34 @@ extern VALUE  rm_no_freeze(VALUE);
 extern char  *rm_string_value_ptr(volatile VALUE *);
 #endif
 extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
+extern int    rm_strcasecmp(const char *, const char *);
 extern void   rm_check_ary_len(VALUE, int);
 extern void   rm_check_frozen(VALUE);
+extern int    rm_check_num2dbl(VALUE);
 extern double rm_fuzz_to_dbl(VALUE);
 extern double rm_percentage(VALUE);
+extern double rm_str_to_pct(VALUE);
 extern VALUE  rm_define_enum_type(char *);
 extern void   rm_write_temp_image(Image *, char *);
 extern void   rm_delete_temp_image(char *);
 extern void   rm_not_implemented(void);
-extern void   rm_handle_error(ExceptionInfo *);
-extern void   rm_handle_all_errors(Image *);
 extern void   rm_attr_write(VALUE, VALUE);
 extern void   rm_get_geometry(VALUE, long *, long *, unsigned long *, unsigned long *, int *);
 extern void   rm_split(Image *);
 
+typedef enum
+{
+    RetainOnError = 0,
+    DestroyOnError = 1
+} ErrorRetention;
+
+extern void   rm_check_image_exception(Image *, ErrorRetention);
+extern void   rm_check_exception(ExceptionInfo *, Image *, ErrorRetention);
+extern void   rm_ensure_result(Image *);
+extern Image *rm_clone_image(Image *);
+#if defined(HAVE_SETIMAGEPROGRESSMONITOR)
+extern MagickBooleanType rm_progress_monitor(const char *, const MagickOffsetType, const MagickSizeType, void *);
+#endif
+
 #endif
 
