@@ -1,6 +1,6 @@
-/* $Id: rmmain.c,v 1.84 2005/04/30 23:01:39 rmagick Exp $ */
+/* $Id: rmmain.c,v 1.120 2006/06/02 23:26:53 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2005 by Timothy P. Hunter
+|                Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmmain.c
 | Author:   Tim Hunter
 | Purpose:  Contains all module, class, method declarations.
@@ -10,12 +10,14 @@
 
 #define MAIN                        // Define external variables
 #include "rmagick.h"
+#include "magick/version.h"
 
 /*----------------------------------------------------------------------------\
 | External declarations
 \----------------------------------------------------------------------------*/
 void Init_RMagick(void);
 
+static void test_Magick_version(void);
 static void version_constants(void);
 
 
@@ -41,7 +43,9 @@ Magick_colors(VALUE class)
     GetExceptionInfo(&exception);
 
     color_ary = GetColorInfoArray(&exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 
     if (rb_block_given_p())
     {
@@ -84,7 +88,9 @@ Magick_colors(VALUE class)
     GetExceptionInfo(&exception);
 
     color_info_list = GetColorInfoList("*", &number_colors, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 #endif
 
 
@@ -172,7 +178,9 @@ Magick_fonts(VALUE class)
 
     GetExceptionInfo(&exception);
     type_info = GetTypeInfoList("*", &number_types, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 #endif
 
     if (rb_block_given_p())
@@ -205,7 +213,8 @@ Magick_fonts(VALUE class)
     GetExceptionInfo(&exception);
 
     type_list = GetTypeInfo("*", &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
 
     // If block, iterate over fonts
     if (rb_block_given_p())
@@ -278,6 +287,9 @@ Magick_init_formats(VALUE class)
 
     GetExceptionInfo(&exception);
     magick_info = (MagickInfo *)GetMagickInfoArray(&exception);
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
     for(m = magick_info; m != NULL; m = m->next)
     {
         rb_hash_aset(formats, rb_str_new2(m->name), MagickInfo_to_format(m));
@@ -305,7 +317,9 @@ Magick_init_formats(VALUE class)
     // IM 6.1.3 added an exception argument to GetMagickInfoList
     GetExceptionInfo(&exception);
     magick_info = GetMagickInfoList("*", &number_formats, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 #endif
 
     for(x = 0; x < number_formats; x++)
@@ -328,7 +342,8 @@ Magick_init_formats(VALUE class)
 
     GetExceptionInfo(&exception);
     m = (MagickInfo *)GetMagickInfo("*", &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
 
     for ( ; m != NULL; m = m->next)
     {
@@ -340,6 +355,98 @@ Magick_init_formats(VALUE class)
 #endif
 }
 
+
+/*
+  Method:   Magick.limit_resource(resource[, limit])
+  Purpose:  Get/set resource limits. If a limit is specified the old limit
+            is set to the new value. Either way the current/new limit is returned.
+  Notes:    Don't support "AreaLimit" because GraphicsMagick doesn't support it.
+*/
+static VALUE
+Magick_limit_resource(int argc, VALUE *argv, VALUE class)
+{
+#if defined(HAVE_GETMAGICKRESOURCELIMIT)
+    volatile VALUE resource, limit;
+    ResourceType res = UndefinedResource;
+    char *str;
+    ID id;
+    unsigned long cur_limit;
+
+    rb_scan_args(argc, argv, "11", &resource, &limit);
+
+    switch (TYPE(resource))
+    {
+        case T_NIL:
+            return class;
+
+        case T_SYMBOL:
+            id = SYM2ID(resource);
+            if (id == rb_intern("memory"))
+            {
+                res = MemoryResource;
+            }
+            else if (id == rb_intern("map"))
+            {
+                res = MapResource;
+            }
+            else if (id == rb_intern("disk"))
+            {
+                res = DiskResource;
+            }
+            else if (id == rb_intern("file"))
+            {
+                res = FileResource;
+            }
+            else
+            {
+                rb_raise(rb_eArgError, "unknown resource: `:%s'", rb_id2name(id));
+            }
+            break;
+
+        default:
+            str = STRING_PTR(resource);
+            if (*str == '\0')
+            {
+                return class;
+            }
+            else if (rm_strcasecmp("memory", str) == 0)
+            {
+                res = MemoryResource;
+            }
+            else if (rm_strcasecmp("map", str) == 0)
+            {
+                res = MapResource;
+            }
+            else if (rm_strcasecmp("disk", str) == 0)
+            {
+                res = DiskResource;
+            }
+            else if (rm_strcasecmp("file", str) == 0)
+            {
+                res = FileResource;
+            }
+            else
+            {
+                rb_raise(rb_eArgError, "unknown resource: `%s'", str);
+            }
+            break;
+    }
+
+    cur_limit = GetMagickResourceLimit(res);
+
+    if (argc > 1)
+    {
+        SetMagickResourceLimit(res, NUM2ULONG(limit));
+    }
+
+    return ULONG2NUM(cur_limit);
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
 /*
     This is the exit known to ImageMagick. Retrieve the monitor
     proc and call it, passing the 3 exit arguments.
@@ -386,7 +493,11 @@ Magick_set_monitor(VALUE class, VALUE monitor)
     {
         Magick_Monitor = rb_intern(MAGICK_MONITOR_CVAR);
         rb_define_class_variable(Module_Magick, MAGICK_MONITOR_CVAR, monitor);
-        ID_call = rb_intern("call");
+
+#if defined(HAVE_SETIMAGEPROGRESSMONITOR)
+        rb_warning("Magick.set_monitor is deprecated; use Image#monitor= or Image::Info#monitor= instead.");
+#endif
+
     }
 
     // If nil, turn off monitoring.
@@ -402,7 +513,7 @@ Magick_set_monitor(VALUE class, VALUE monitor)
         // available, use the 3-parm version of rb_cvar_set.
         RUBY18(rb_cvar_set(Module_Magick, Magick_Monitor, monitor, 0);)
         RUBY16(rb_cvar_set(Module_Magick, Magick_Monitor, monitor);)
-        (void) SetMonitorHandler(&monitor_handler);
+        (void) SetMonitorHandler((MonitorHandler)&monitor_handler);
     }
 
     return class;
@@ -466,13 +577,8 @@ Magick_set_log_event_mask(int argc, VALUE *argv, VALUE class)
 static VALUE
 Magick_set_log_format(VALUE class, VALUE format)
 {
-#ifdef HAVE_SETLOGFORMAT
     SetLogFormat(STRING_PTR(format));
     return class;
-#else
-    rm_not_implemented();
-    return (VALUE)0;
-#endif
 }
 
 
@@ -490,6 +596,7 @@ Init_RMagick(void)
     volatile VALUE observable;
 
     InitializeMagick("RMagick");
+    test_Magick_version();
 
     Module_Magick = rb_define_module("Magick");
 
@@ -498,6 +605,7 @@ Init_RMagick(void)
     /*-----------------------------------------------------------------------*/
 
     ID__dummy_img_      = rb_intern("_dummy_img_");
+    ID_call             = rb_intern("call");
     ID_changed          = rb_intern("changed");
     ID_cur_image        = rb_intern("cur_image");
     ID_dup              = rb_intern("dup");
@@ -527,6 +635,7 @@ Init_RMagick(void)
     rb_define_module_function(Module_Magick, "colors", Magick_colors, 0);
     rb_define_module_function(Module_Magick, "fonts", Magick_fonts, 0);
     rb_define_module_function(Module_Magick, "init_formats", Magick_init_formats, 0);
+    rb_define_module_function(Module_Magick, "limit_resource", Magick_limit_resource, -1);
     rb_define_module_function(Module_Magick, "set_monitor", Magick_set_monitor, 1);
     rb_define_module_function(Module_Magick, "set_cache_threshold", Magick_set_cache_threshold, 1);
     rb_define_module_function(Module_Magick, "set_log_event_mask", Magick_set_log_event_mask, -1);
@@ -594,6 +703,7 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Image, matte_color)
     DCL_ATTR_READER(Image, mean_error_per_pixel)
     DCL_ATTR_READER(Image, mime_type)
+    DCL_ATTR_WRITER(Image, monitor)
     DCL_ATTR_ACCESSOR(Image, montage)
     DCL_ATTR_READER(Image, normalized_mean_error)
     DCL_ATTR_READER(Image, normalized_maximum_error)
@@ -611,14 +721,17 @@ Init_RMagick(void)
     DCL_ATTR_READER(Image, scene)
     DCL_ATTR_ACCESSOR(Image, start_loop)
     DCL_ATTR_ACCESSOR(Image, class_type)
+    DCL_ATTR_ACCESSOR(Image, ticks_per_second)
     DCL_ATTR_ACCESSOR(Image, tile_info)
     DCL_ATTR_READER(Image, total_colors)
     DCL_ATTR_ACCESSOR(Image, units)
+    DCL_ATTR_ACCESSOR(Image, virtual_pixel_method)
     DCL_ATTR_ACCESSOR(Image, x_resolution)
     DCL_ATTR_ACCESSOR(Image, y_resolution)
 
     rb_define_method(Class_Image, "adaptive_threshold", Image_adaptive_threshold, -1);
     rb_define_method(Class_Image, "add_noise", Image_add_noise, 1);
+    rb_define_method(Class_Image, "add_noise_channel", Image_add_noise_channel, -1);
     rb_define_method(Class_Image, "affine_transform", Image_affine_transform, 1);
     rb_define_method(Class_Image, "[]", Image_aref, 1);
     rb_define_method(Class_Image, "[]=", Image_aset, 2);
@@ -652,6 +765,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "composite_affine", Image_composite_affine, 2);
     rb_define_method(Class_Image, "compress_colormap!", Image_compress_colormap_bang, 0);
     rb_define_method(Class_Image, "contrast", Image_contrast, -1);
+    rb_define_method(Class_Image, "contrast_stretch_channel", Image_contrast_stretch_channel, -1);
     rb_define_method(Class_Image, "convolve", Image_convolve, 2);
     rb_define_method(Class_Image, "convolve_channel", Image_convolve_channel, -1);
     rb_define_method(Class_Image, "copy", Image_copy, 0);
@@ -662,6 +776,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "difference", Image_difference, 1);
     rb_define_method(Class_Image, "dispatch", Image_dispatch, -1);
     rb_define_method(Class_Image, "display", Image_display, 0);
+    rb_define_method(Class_Image, "distortion_channel", Image_distortion_channel, -1);
     rb_define_method(Class_Image, "_dump", Image__dump, 1);
     rb_define_method(Class_Image, "dup", Image_dup, 0);
     rb_define_method(Class_Image, "each_profile", Image_each_profile, 0);
@@ -670,7 +785,8 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "enhance", Image_enhance, 0);
     rb_define_method(Class_Image, "equalize", Image_equalize, 0);
     rb_define_method(Class_Image, "erase!", Image_erase_bang, 0);
-    rb_define_method(Class_Image, "export_pixels", Image_export_pixels, 5);
+    rb_define_method(Class_Image, "export_pixels", Image_export_pixels, -1);
+    rb_define_method(Class_Image, "export_pixels_to_str", Image_export_pixels_to_str, -1);
     rb_define_method(Class_Image, "flip", Image_flip, 0);
     rb_define_method(Class_Image, "flip!", Image_flip_bang, 0);
     rb_define_method(Class_Image, "flop", Image_flop, 0);
@@ -685,10 +801,10 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "grey?", Image_gray_q, 0);
     rb_define_method(Class_Image, "grayscale_pseudo_class", Image_grayscale_pseudo_class, -1);
     rb_define_method(Class_Image, "implode", Image_implode, -1);
-    rb_define_method(Class_Image, "import_pixels", Image_import_pixels, 6);
+    rb_define_method(Class_Image, "import_pixels", Image_import_pixels, -1);
     rb_define_method(Class_Image, "initialize_copy", Image_init_copy, 1);
     rb_define_method(Class_Image, "inspect", Image_inspect, 0);
-    rb_define_method(Class_Image, "level", Image_level, -1);
+    rb_define_method(Class_Image, "level2", Image_level2, -1);
     rb_define_method(Class_Image, "level_channel", Image_level_channel, -1);
     rb_define_method(Class_Image, "magnify", Image_magnify, 0);
     rb_define_method(Class_Image, "magnify!", Image_magnify_bang, 0);
@@ -707,7 +823,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "oil_paint", Image_oil_paint, -1);
     rb_define_method(Class_Image, "opaque", Image_opaque, 2);
     rb_define_method(Class_Image, "opaque?", Image_opaque_q, 0);
-    rb_define_method(Class_Image, "ordered_dither", Image_ordered_dither, 0);
+    rb_define_method(Class_Image, "ordered_dither", Image_ordered_dither, -1);
     rb_define_method(Class_Image, "palette?", Image_palette_q, 0);
     rb_define_method(Class_Image, "pixel_color", Image_pixel_color, -1);
     rb_define_method(Class_Image, "posterize", Image_posterize, -1);
@@ -764,6 +880,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "trim!", Image_trim_bang, 0);
     rb_define_method(Class_Image, "unsharp_mask", Image_unsharp_mask, -1);
     rb_define_method(Class_Image, "unsharp_mask_channel", Image_unsharp_mask_channel, -1);
+    rb_define_method(Class_Image, "vignette", Image_vignette, -1);
     rb_define_method(Class_Image, "wave", Image_wave, -1);
     rb_define_method(Class_Image, "white_threshold", Image_white_threshold, -1);
     rb_define_method(Class_Image, "write", Image_write, 1);
@@ -778,7 +895,7 @@ Init_RMagick(void)
     rb_define_alias(Class_ImageList, "__display__", "display");
 
     // Define an alias for Array's "map" method.
-    rb_define_alias(Class_ImageList, "__map__", "map");
+    rb_define_alias(Class_ImageList, "__ary_map__", "map");
 
     rb_define_method(Class_ImageList, "animate", ImageList_animate, -1);
     rb_define_method(Class_ImageList, "append", ImageList_append, 1);
@@ -791,6 +908,7 @@ Init_RMagick(void)
     rb_define_method(Class_ImageList, "montage", ImageList_montage, 0);
     rb_define_method(Class_ImageList, "morph", ImageList_morph, 1);
     rb_define_method(Class_ImageList, "mosaic", ImageList_mosaic, 0);
+    rb_define_method(Class_ImageList, "optimize_layers", ImageList_optimize_layers, 1);
     rb_define_method(Class_ImageList, "quantize", ImageList_quantize, -1);
     rb_define_method(Class_ImageList, "to_blob", ImageList_to_blob, 0);
     rb_define_method(Class_ImageList, "write", ImageList_write, 1);
@@ -808,6 +926,7 @@ Init_RMagick(void)
 
     DCL_ATTR_WRITER(Draw, affine)
     DCL_ATTR_WRITER(Draw, align)
+    DCL_ATTR_WRITER(Draw, border_color)
     DCL_ATTR_WRITER(Draw, decorate)
     DCL_ATTR_WRITER(Draw, density)
     DCL_ATTR_WRITER(Draw, encoding)
@@ -866,6 +985,13 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Pixel, blue)
     DCL_ATTR_ACCESSOR(Pixel, opacity)
 
+    // Define the CMYK attributes
+    DCL_ATTR_ACCESSOR(Pixel, cyan)
+    DCL_ATTR_ACCESSOR(Pixel, magenta)
+    DCL_ATTR_ACCESSOR(Pixel, yellow)
+    DCL_ATTR_ACCESSOR(Pixel, black)
+
+
     // Define the instance methods
     rb_define_method(Class_Pixel, "<=>", Pixel_spaceship, 1);
     rb_define_method(Class_Pixel, "===", Pixel_case_eq, 1);
@@ -933,25 +1059,36 @@ Init_RMagick(void)
     rb_define_method(Class_Info, "undefine", Info_undefine, 2);
 
     DCL_ATTR_ACCESSOR(Info, antialias)
+    DCL_ATTR_ACCESSOR(Info, authenticate)
     DCL_ATTR_ACCESSOR(Info, background_color)
     DCL_ATTR_ACCESSOR(Info, border_color)
     DCL_ATTR_ACCESSOR(Info, colorspace)
+    DCL_ATTR_ACCESSOR(Info, comment)        // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, compression)
+    DCL_ATTR_ACCESSOR(Info, delay)          // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, density)
     DCL_ATTR_ACCESSOR(Info, depth)
+    DCL_ATTR_ACCESSOR(Info, dispose)        // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, dither)
     DCL_ATTR_ACCESSOR(Info, extract)    // new in 5.5.6, replaces tile
     DCL_ATTR_ACCESSOR(Info, filename)
+    DCL_ATTR_ACCESSOR(Info, fill)
     DCL_ATTR_ACCESSOR(Info, font)
     DCL_ATTR_ACCESSOR(Info, format)
     DCL_ATTR_ACCESSOR(Info, fuzz)
+    DCL_ATTR_ACCESSOR(Info, gravity)
     DCL_ATTR_ACCESSOR(Info, group)
     DCL_ATTR_ACCESSOR(Info, interlace)
+    DCL_ATTR_ACCESSOR(Info, label)          // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, matte_color)
+    DCL_ATTR_WRITER(Info, monitor)
     DCL_ATTR_ACCESSOR(Info, monochrome)
     DCL_ATTR_ACCESSOR(Info, number_scenes)  // new in 5.5.6, replaces subrange
+    DCL_ATTR_ACCESSOR(Info, orientation)    // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, page)
+    DCL_ATTR_ACCESSOR(Info, pointsize)
     DCL_ATTR_ACCESSOR(Info, quality)
+    DCL_ATTR_ACCESSOR(Info, sampling_factor)
     DCL_ATTR_ACCESSOR(Info, scene)      // new in 5.5.6, replaces subimage
     DCL_ATTR_ACCESSOR(Info, server_name)
     DCL_ATTR_ACCESSOR(Info, size)
@@ -1060,6 +1197,9 @@ Init_RMagick(void)
 #if defined(HAVE_INDEXCHANNEL)
         ENUMERATOR(IndexChannel)    // 5.5.8
 #endif
+#if defined(HAVE_GRAYCHANNEL)
+        ENUMERATOR(GrayChannel)
+#endif
 #if defined(HAVE_ALLCHANNELS)
         ENUMERATOR(AllChannels)
 #endif
@@ -1090,14 +1230,31 @@ Init_RMagick(void)
                       , rm_enum_new(Class_ColorspaceType
                       , ID2SYM(rb_intern("SRGBColorspace"))
                       , INT2FIX(sRGBColorspace)));
-#if defined(HAVE_HSLCOLORSPACE)
-        ENUMERATOR(HSLColorspace)       // 5.5.7
-#endif
-#if defined(HAVE_HWBCOLORSPACE)
-        ENUMERATOR(HWBColorspace)       // 5.5.7
-#endif
+        ENUMERATOR(HSLColorspace)
+        ENUMERATOR(HWBColorspace)
 #if defined(HAVE_HSBCOLORSPACE)
-        ENUMERATOR(HSBColorspace)       // 6.0.0
+        ENUMERATOR(HSBColorspace)           // IM 6.0.0
+#endif
+#if defined(HAVE_CINEONLOGRGBCOLORSPACE)
+        ENUMERATOR(CineonLogRGBColorspace)  // GM 1.2
+#endif
+#if defined(HAVE_LABCOLORSPACE)
+        ENUMERATOR(LABColorspace)           // GM 1.2
+#endif
+#if defined(HAVE_REC601LUMACOLORSPACE)
+        ENUMERATOR(Rec601LumaColorspace)    // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_REC601YCBCRCOLORSPACE)
+        ENUMERATOR(Rec601YCbCrColorspace)   // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_REC709LUMACOLORSPACE)
+        ENUMERATOR(Rec709LumaColorspace)    // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_REC709YCBCRCOLORSPACE)
+        ENUMERATOR(Rec709YCbCrColorspace)   // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_LOGCOLORSPACE)
+        ENUMERATOR(LogColorspace)           // IM 6.2.3
 #endif
     END_ENUM
 
@@ -1111,9 +1268,7 @@ Init_RMagick(void)
                   , ID2SYM(rb_intern("AllCompliance"))
                   , INT2FIX(SVGCompliance|X11Compliance|XPMCompliance)));
 
-#if defined(HAVE_NOCOMPLIANCE)
         ENUMERATOR(NoCompliance)
-#endif
         ENUMERATOR(SVGCompliance)
         ENUMERATOR(X11Compliance)
         ENUMERATOR(XPMCompliance)
@@ -1178,6 +1333,8 @@ Init_RMagick(void)
         ENUMERATOR(XorCompositeOp)
 
 #if defined(HAVE_COLORDODGECOMPOSITEOP)
+        ENUMERATOR(BlendCompositeOp)
+        ENUMERATOR(ColorBurnCompositeOp)
         ENUMERATOR(ColorDodgeCompositeOp)
         ENUMERATOR(ExclusionCompositeOp)
         ENUMERATOR(HardLightCompositeOp)
@@ -1198,7 +1355,8 @@ Init_RMagick(void)
 #endif
         ENUMERATOR(LosslessJPEGCompression)
         ENUMERATOR(LZWCompression)
-        ENUMERATOR(RunlengthEncodedCompression)
+        ENUMERATOR(RLECompression)              // preferred
+        ENUMERATOR(RunlengthEncodedCompression) // deprecated
         ENUMERATOR(ZipCompression)
     END_ENUM
 
@@ -1210,15 +1368,13 @@ Init_RMagick(void)
         ENUMERATOR(LineThroughDecoration)
     END_ENUM
 
-#if defined(HAVE_DISPOSETYPE)
-    // DisposeType constants (5.5.1)
+    // DisposeType constants
     DEF_ENUM(DisposeType)
         ENUMERATOR(UndefinedDispose)
         ENUMERATOR(BackgroundDispose)
         ENUMERATOR(NoneDispose)
         ENUMERATOR(PreviousDispose)
     END_ENUM
-#endif
 
     DEF_ENUM(EndianType)
         ENUMERATOR(UndefinedEndian)
@@ -1248,6 +1404,13 @@ Init_RMagick(void)
 
     // GravityType constants
     DEF_ENUM(GravityType)
+#if defined(HAVE_UNDEFINEDGRAVITY)
+        ENUMERATOR(UndefinedGravity)
+#else
+        // Provide this enumerator in older (pre 6.0.0) versions of ImageMagick
+        _enum = rm_enum_new(_cls, ID2SYM(rb_intern("UndefinedGravity")), INT2FIX(0));\
+        rb_define_const(Module_Magick, "UndefinedGravity", _enum);
+#endif
         ENUMERATOR(ForgetGravity)
         ENUMERATOR(NorthWestGravity)
         ENUMERATOR(NorthGravity)
@@ -1285,6 +1448,22 @@ Init_RMagick(void)
         ENUMERATOR(PartitionInterlace)
     END_ENUM
 
+#if defined(HAVE_COMPAREIMAGELAYERS)
+    DEF_ENUM(MagickLayerMethod)
+        ENUMERATOR(UndefinedLayer)
+        ENUMERATOR(CompareAnyLayer)
+        ENUMERATOR(CompareClearLayer)
+        ENUMERATOR(CompareOverlayLayer)
+        ENUMERATOR(OptimizeLayer)
+        ENUMERATOR(OptimizePlusLayer)
+#if defined(HAVE_COALESCELAYER)
+        // Introduced in IM 6.2.7-0
+        ENUMERATOR(CoalesceLayer)
+        ENUMERATOR(DisposeLayer)
+#endif
+    END_ENUM
+#endif
+
 #if defined(HAVE_COMPAREIMAGECHANNELS)
     DEF_ENUM(MetricType)
         ENUMERATOR(UndefinedMetric)
@@ -1395,6 +1574,22 @@ Init_RMagick(void)
         ENUMERATOR(PixelsPerCentimeterResolution)
     END_ENUM
 
+    // StorageType
+    DEF_ENUM(StorageType)
+#if defined(HAVE_UNDEFINEDGRAVITY)      // UndefinedGravity & UndefinedPixel were both introduced in IM 6.0.0
+        ENUMERATOR(UndefinedPixel)
+#endif
+        ENUMERATOR(CharPixel)
+        ENUMERATOR(DoublePixel)
+        ENUMERATOR(FloatPixel)
+        ENUMERATOR(IntegerPixel)
+        ENUMERATOR(LongPixel)
+#if defined(HAVE_QUANTUMPIXEL)
+        ENUMERATOR(QuantumPixel)
+#endif
+        ENUMERATOR(ShortPixel)
+    END_ENUM
+
     // StretchType constants
     DEF_ENUM(StretchType)
         ENUMERATOR(NormalStretch)
@@ -1417,6 +1612,18 @@ Init_RMagick(void)
         ENUMERATOR(AnyStyle)
     END_ENUM
 
+    // VirtualPixelMethod
+    DEF_ENUM(VirtualPixelMethod)
+        ENUMERATOR(UndefinedVirtualPixelMethod)
+        ENUMERATOR(EdgeVirtualPixelMethod)
+        ENUMERATOR(MirrorVirtualPixelMethod)
+        ENUMERATOR(TileVirtualPixelMethod)
+#if defined(HAVE_TRANSPARENTVIRTUALPIXELMETHOD)
+        ENUMERATOR(BackgroundVirtualPixelMethod)
+        ENUMERATOR(TransparentVirtualPixelMethod)
+#endif
+    END_ENUM
+
     // WeightType constants
     DEF_ENUM(WeightType)
         ENUMERATOR(AnyWeight)
@@ -1500,6 +1707,64 @@ Init_RMagick(void)
 
 }
 
+
+
+
+/*
+ *  Static:     test_Magick_version
+ *  Purpose:    Ensure the version of ImageMagick we're running with matches
+ *              the version we were compiled with.
+ *  Notes:      Bypass the test by defining the constant RMAGICK_BYPASS_VERSION_TEST
+ *              to 'true' at the top level, before requiring 'RMagick'
+*/
+static void test_Magick_version(void)
+{
+    unsigned long version_number;
+    const char *version_str;
+    const char *web_site =
+#if defined(MagickHomeURL)
+        MagickHomeURL
+#else
+        MagickWebSite
+#endif
+        ;
+    int x, n;
+    ID bypass = rb_intern("RMAGICK_BYPASS_VERSION_TEST");
+
+    if (RTEST(rb_const_defined(rb_cObject, bypass)) && RTEST(rb_const_get(rb_cObject, bypass)))
+    {
+        return;
+    }
+
+    version_str = GetMagickVersion(&version_number);
+    if (version_number != MagickLibVersion)
+    {
+        // Extract the string "ImageMagick X.Y.Z"
+        n = 0;
+        for (x = 0; version_str[x] != '\0'; x++)
+        {
+            if (version_str[x] == ' ' && ++n == 2)
+            {
+                break;
+            }
+        }
+
+        rb_raise(rb_eRuntimeError,
+            "This version of RMagick was created to run with %s %s\n"
+            "but %.*s is installed on this system. You should either\n"
+            "   1) refer to the RMagick README file to learn how to create\n"
+            "      a version of RMagick for %.*s, or\n"
+            "   2) download %s %s from %s and install it." ,
+            MagickPackageName, MagickLibVersionText, x, version_str, x, version_str,
+            MagickPackageName, MagickLibVersionText, web_site);
+    }
+
+}
+
+
+
+
+
 /*
     Static:     version_constants
     Purpose:    create Version, Magick_version, and Version_long constants.
@@ -1507,19 +1772,29 @@ Init_RMagick(void)
 static void version_constants(void)
 {
     const char *mgk_version;
-    char long_version[400];
+    volatile VALUE str;
+    char long_version[1000];
 
     mgk_version = GetMagickVersion(NULL);
-    rb_define_const(Module_Magick, "Magick_version", rb_str_new2(mgk_version));
 
-    rb_define_const(Module_Magick, "Version", rb_str_new2(PACKAGE_STRING));
+    str = rb_str_new2(mgk_version);
+    rb_obj_freeze(str);
+    rb_define_const(Module_Magick, "Magick_version", str);
+
+    str = rb_str_new2(PACKAGE_STRING);
+    rb_obj_freeze(str);
+    rb_define_const(Module_Magick, "Version", str);
+
     sprintf(long_version,
-        "This is %s ($Date: 2005/04/30 23:01:39 $) Copyright (C) 2005 by Timothy P. Hunter\n"
+        "This is %s ($Date: 2006/06/02 23:26:53 $) Copyright (C) 2006 by Timothy P. Hunter\n"
         "Built with %s\n"
         "Built for %s\n"
         "Web page: http://rmagick.rubyforge.org\n"
         "Email: rmagick@rubyforge.org\n",
         PACKAGE_STRING, mgk_version, RUBY_VERSION_STRING);
-    rb_define_const(Module_Magick, "Long_version", rb_str_new2(long_version));
+
+    str = rb_str_new2(long_version);
+    rb_obj_freeze(str);
+    rb_define_const(Module_Magick, "Long_version", str);
 
 }
