@@ -1,6 +1,6 @@
-# $Id: RMagick.rb,v 1.23 2005/03/05 16:19:10 rmagick Exp $
+# $Id: RMagick.rb,v 1.42 2006/06/03 22:56:40 rmagick Exp $
 #==============================================================================
-#                  Copyright (C) 2005 by Timothy P. Hunter
+#                  Copyright (C) 2006 by Timothy P. Hunter
 #   Name:       RMagick.rb
 #   Author:     Tim Hunter
 #   Purpose:    Extend Ruby to interface with ImageMagick.
@@ -164,6 +164,17 @@ class Draw
         AnyStyle.to_i => 'all'
         }
 
+  private
+    def enquote(str)
+        if str.length > 2 && /\A(?:\"[^\"]+\"|\'[^\']+\'|\{[^\}]+\})\z/.match(str)
+            return str
+        else
+            return '"' + str + '"'
+        end
+    end
+
+  public
+
     # Apply coordinate transformations to support scaling (s), rotation (r),
     # and translation (t). Angles are specified in radians.
     def affine(sx, rx, ry, sy, tx, ty)
@@ -227,7 +238,7 @@ class Draw
         if ( DECORATION_TYPE_NAMES.has_key?(decoration.to_i) )
             primitive "decorate #{DECORATION_TYPE_NAMES[decoration.to_i]}"
         else
-            primitive "decorate #{decoration}"
+            primitive "decorate #{enquote(decoration)}"
         end
     end
 
@@ -263,7 +274,7 @@ class Draw
 
     # Specify object fill, a color name or pattern name
     def fill(colorspec)
-        primitive "fill #{colorspec}"
+        primitive "fill #{enquote(colorspec)}"
     end
     alias fill_color fill
     alias fill_pattern fill
@@ -463,7 +474,7 @@ class Draw
 
     # Specify the object stroke, a color name or pattern name.
     def stroke(colorspec)
-        primitive "stroke #{colorspec}"
+        primitive "stroke #{enquote(colorspec)}"
     end
     alias stroke_color stroke
     alias stroke_pattern stroke
@@ -525,13 +536,23 @@ class Draw
         primitive "stroke-width #{pixels}"
     end
 
-    # Draw text at position x,y. Generally it's best to surround the text with
-    # quotes. For example,
-    #      gc.text(100, 100, "'embedded blanks'")
+    # Draw text at position x,y. Add quotes to text that is not already quoted.
     def text(x, y, text)
         if text.to_s.empty?
             raise ArgumentError, "missing text argument"
         end
+        if text.length > 2 && /\A(?:\"[^\"]+\"|\'[^\']+\'|\{[^\}]+\})\z/.match(text)
+            ; # text already quoted
+        elsif !text['\'']
+            text = '\''+text+'\''
+        elsif !text['"']
+            text = '"'+text+'"'
+        elsif !(text['{'] || text['}'])
+            text = '{'+text+'}'
+        else
+            # escape existing braces, surround with braces
+            text = '{' +  text.gsub(/[}]/) { |b| '\\' + b } + '}'
+        end
         primitive "text #{x},#{y} #{text}"
     end
 
@@ -559,7 +580,7 @@ class Draw
 
     # Specify color underneath text
     def text_undercolor(color)
-        primitive "text-undercolor #{color}"
+        primitive "text-undercolor #{enquote(color)}"
     end
 
     # Specify center of coordinate space to use for subsequent drawing
@@ -616,66 +637,24 @@ class Image
         self
     end
 
-    # Used by ImageList methods - see ImageList#cur_image
-    def cur_image
+    # Force an image to exact dimensions without changing the aspect ratio.
+    # Resize and crop if necessary. (Thanks to Jerett Taylor!)
+    def crop_resized(ncols, nrows, gravity=CenterGravity)
+        copy.crop_resized!(ncols, nrows, gravity)
+    end
+
+    def crop_resized!(ncols, nrows, gravity=CenterGravity)
+        if ncols != columns || nrows != rows
+            scale = [ncols/columns.to_f, nrows/rows.to_f].max
+            resize!(scale*columns+0.5, scale*rows+0.5)
+        end
+        crop!(gravity, ncols, nrows, true) if ncols != columns || nrows != rows
         self
     end
 
-    # These four methods are equivalent to the Draw#matte
-    # method with the "Point", "Replace", "Floodfill", "FilltoBorder", and
-    # "Replace" arguments, respectively.
-
-    # Make the pixel at (x,y) transparent.
-    def matte_point(x, y)
-        f = copy
-        f.opacity = OpaqueOpacity unless f.matte
-        pixel = f.pixel_color(x,y)
-        pixel.opacity = TransparentOpacity
-        f.pixel_color(x, y, pixel)
-        return f
-    end
-
-    # Make transparent all pixels that are the same color as the
-    # pixel at (x, y).
-    def matte_replace(x, y)
-        f = copy
-        f.opacity = OpaqueOpacity unless f.matte
-        target = f.pixel_color(x, y)
-        f.transparent(target)
-    end
-
-    # Make transparent any pixel that matches the color of the pixel
-    # at (x,y) and is a neighbor.
-    def matte_floodfill(x, y)
-        f = copy
-        f.opacity = OpaqueOpacity unless f.matte
-        target = f.pixel_color(x, y)
-        f.matte_flood_fill(target, TransparentOpacity,
-                           x, y, FloodfillMethod)
-    end
-
-    # Make transparent any neighbor pixel that is not the border color.
-    def matte_fill_to_border(x, y)
-        f = copy
-        f.opacity = Magick::OpaqueOpacity unless f.matte
-        f.matte_flood_fill(border_color, TransparentOpacity,
-                           x, y, FillToBorderMethod)
-    end
-
-    # Make all pixels transparent.
-    def matte_reset!
-        self.opacity = Magick::TransparentOpacity
-    end
-
-    # Replace matching neighboring pixels with texture pixels
-    def texture_floodfill(x, y, texture)
-        target = pixel_color(x, y)
-        texture_flood_fill(target, texture, x, y, FloodfillMethod)
-    end
-
-    # Replace neighboring pixels to border color with texture pixels
-    def texture_fill_to_border(x, y, texture)
-        texture_flood_fill(border_color, texture, x, y, FillToBorderMethod)
+    # Used by ImageList methods - see ImageList#cur_image
+    def cur_image
+        self
     end
 
     # Retrieve EXIF data by entry or all. If one or more entry names specified,
@@ -719,6 +698,106 @@ class Image
         return hash
     end
 
+    # Patches problematic change to the order of arguments in 1.11.0.
+    # Before this release, the order was
+    #       black_point, gamma, white_point
+    # RMagick 1.11.0 changed this to
+    #       black_point, white_point, gamma
+    # This fix tries to determine if the arguments are in the old order and
+    # if so, swaps the gamma and white_point arguments.  Then it calls
+    # level2, which simply accepts the arguments as given.
+
+    # Inspect the gamma and white point values and swap them if they
+    # look like they're in the old order.
+
+    # (Thanks to Al Evans for the suggestion.)
+    def level(black_point=0.0, white_point=nil, gamma=nil)
+        white_point ||= Magick::MaxRGB - black_point
+        gamma_arg = gamma
+        gamma ||= 1.0
+
+        if gamma.abs > 10.0 || white_point.abs <= 10.0 || white_point.abs < gamma.abs
+            gamma, white_point = white_point, gamma
+            unless gamma_arg
+                white_point = Magick::MaxRGB - black_point
+            end
+        end
+
+        return level2(black_point, white_point, gamma)
+    end
+
+    # These four methods are equivalent to the Draw#matte method
+    # with the "Point", "Replace", "Floodfill", "FilltoBorder", and
+    # "Replace" arguments, respectively.
+
+    # Make the pixel at (x,y) transparent.
+    def matte_point(x, y)
+        f = copy
+        f.opacity = OpaqueOpacity unless f.matte
+        pixel = f.pixel_color(x,y)
+        pixel.opacity = TransparentOpacity
+        f.pixel_color(x, y, pixel)
+        return f
+    end
+
+    # Make transparent all pixels that are the same color as the
+    # pixel at (x, y).
+    def matte_replace(x, y)
+        f = copy
+        f.opacity = OpaqueOpacity unless f.matte
+        target = f.pixel_color(x, y)
+        f.transparent(target)
+    end
+
+    # Make transparent any pixel that matches the color of the pixel
+    # at (x,y) and is a neighbor.
+    def matte_floodfill(x, y)
+        f = copy
+        f.opacity = OpaqueOpacity unless f.matte
+        target = f.pixel_color(x, y)
+        f.matte_flood_fill(target, TransparentOpacity,
+                           x, y, FloodfillMethod)
+    end
+
+    # Make transparent any neighbor pixel that is not the border color.
+    def matte_fill_to_border(x, y)
+        f = copy
+        f.opacity = Magick::OpaqueOpacity unless f.matte
+        f.matte_flood_fill(border_color, TransparentOpacity,
+                           x, y, FillToBorderMethod)
+    end
+
+    # Make all pixels transparent.
+    def matte_reset!
+        self.opacity = Magick::TransparentOpacity
+        self
+    end
+
+    # Convenience method to resize retaining the aspect ratio.
+    # (Thanks to Robert Manni!)
+    def resize_to_fit(cols, rows)
+        change_geometry(Geometry.new(cols, rows)) do |ncols, nrows|
+            resize(ncols, nrows)
+        end
+    end
+
+    def resize_to_fit!(cols, rows)
+        change_geometry(Geometry.new(cols, rows)) do |ncols, nrows|
+            resize!(ncols, nrows)
+        end
+    end
+
+    # Replace matching neighboring pixels with texture pixels
+    def texture_floodfill(x, y, texture)
+        target = pixel_color(x, y)
+        texture_flood_fill(target, texture, x, y, FloodfillMethod)
+    end
+
+    # Replace neighboring pixels to border color with texture pixels
+    def texture_fill_to_border(x, y, texture)
+        texture_flood_fill(border_color, texture, x, y, FillToBorderMethod)
+    end
+
     # Construct a view. If a block is present, yield and pass the view
     # object, otherwise return the view object.
     def view(x, y, width, height)
@@ -742,6 +821,12 @@ class Image
         attr_accessor :dirty
 
         def initialize(img, x, y, width, height)
+            if width <= 0 || height <= 0
+                raise ArgumentError, "invalid geometry (#{width}x#{height}+#{x}+#{y})"
+            end
+            if x < 0 || y < 0 || (x+width) > img.columns || (y+height) > img.rows
+                raise RangeError, "geometry (#{width}x#{height}+#{x}+#{y}) exceeds image boundary"
+            end
             @view = img.get_pixels(x, y, width, height)
             @img = img
             @x = x
@@ -988,11 +1073,16 @@ protected
         unless obj.kind_of? Magick::Image
             raise ArgumentError, "Magick::Image required (#{obj.class} given)"
         end
+        true
     end
 
     # Ensure array is always an array of Magick::Image objects
     def is_a_image_array(ary)
+        unless ary.respond_to? :each
+            raise ArgumentError, "Magick::ImageList or array of Magick::Images required (#{ary.class} given)"
+        end
         ary.each { |obj| is_a_image obj }
+        true
     end
 
     # Find old current image, update @scene
@@ -1020,12 +1110,20 @@ public
 
     # Allow scene to be set to nil
     def scene=(n)
-        if (length == 0 && n != nil) || (length > 0 && n == nil)
-            raise IndexError, "scene number out of bounds"
-        elsif (length == 0) || (Integer(n) < 0) || (Integer(n) > length - 1)
+        if n.nil?
+            raise IndexError, "scene number out of bounds" unless length == 0
+            @scene = nil
+            return @scene
+        elsif length == 0
             raise IndexError, "scene number out of bounds"
         end
-        @scene = Integer(n)
+
+        n = Integer(n)
+        if n < 0 || n > length - 1
+            raise IndexError, "scene number out of bounds"
+        end
+        @scene = n
+        return @scene
     end
 
     def [](*args)
@@ -1038,15 +1136,17 @@ public
 
     def []=(*args)
         if args.length == 3             # f[start,length] = [f1,f2...]
-            is_a_image_array args[2]
+            args[2].kind_of?(Magick::Image) || is_a_image_array(args[2])
             super
-            if args[1] > 0
-                @scene = args[0] + args[1] - 1
-            else                        # inserts elements if length == 0
+            args[0] = args[0] + length if (args[0] < 0)
+            args[1] = length - args[0] if (args[0] + args[1] > length)
+            if args[2].kind_of?(Magick::Image)
+                @scene = args[0]
+            else
                 @scene = args[0] + args[2].length - 1
             end
         elsif args[0].kind_of? Range    # f[first..last] = [f1,f2...]
-            is_a_image_array args[1]
+            args[1].kind_of?(Magick::Image) || is_a_image_array(args[1])
             super
             @scene = args[0].end
         else                            # f[index] = f1
@@ -1166,49 +1266,61 @@ public
         return a
     end
 
-    def fill(obj, *args)
-        is_a_image obj
+    def fill(*args, &block)
+        is_a_image args[0] unless block_given?
+        cfid = self[@scene].__id__ rescue nil
+        super
+        is_a_image_array self
+        set_cf cfid
+        return self
+    end
+
+    def find_all(&block)
         cfid = self[@scene].__id__ rescue nil
         a = super
-        set_cf cfid
+        a.set_cf cfid
         return a
     end
 
-    if Array.instance_methods(false).include? 'fetch' then
-        def fetch(*args,&block)
-            super
-        end
-    end
-
-    if Array.instance_methods(false).include? 'insert' then
+    if self.superclass.instance_methods(true).include? 'insert' then
         def insert(*args)
+            raise(ArgumentError, "can't insert nil") unless args.length > 1
+            is_a_image_array args[1,args.length-1]
             cfid = self[@scene].__id__ rescue nil
-            a = self.class.new.replace super
-            a.set_cf cfid
-            return a
+            super
+            set_cf cfid
+            return self
         end
     end
 
-    # __map__ is a synonym for Array#map. We've used an alias
-    # so it doesn't conflict with our own map method.
-    if Array.instance_methods(false).include? '__map__' then
-        def __map__(&block)
-            cfid = self[@scene].__id__ rescue nil
-            a = self.class.new.replace super
-            a.set_cf cfid
-            return a
+    # Enumerable (or Array) has a #map method that conflicts with our
+    # own #map method. RMagick.so has defined a synonym for that #map
+    # called Array#__ary_map__. Here, we define Magick::ImageList#__map__
+    # to allow the use of the Enumerable/Array#map method on ImageList objects.
+    def __map__(&block)
+        cfid = self[@scene].__id__ rescue nil
+        ensure_image = Proc.new do |img|
+            rv = block.call(img)
+            is_a_image rv
+            return rv
         end
+        a = self.class.new.replace __ary_map__(&ensure_image)
+        a.set_cf cfid
+        return a
     end
 
     def map!(&block)
-        super
-        is_a_image_array self
-        self
+        ensure_image = Proc.new do |img|
+            rv = block.call(img)
+            is_a_image rv
+            return rv
+        end
+        super(&ensure_image)
     end
 
     def pop
         cfid = self[@scene].__id__ rescue nil
-        a = super
+        a = super       # can return nil
         set_cf cfid
         return a
     end
@@ -1220,18 +1332,16 @@ public
         self
     end
 
-    if Array.instance_methods(false).include? 'reject' then
-        def reject(&block)
-            cfid = self[@scene].__id__ rescue nil
-            a = super
-            set_cf cfid
-            return a
-        end
+    def reject(&block)
+        cfid = self[@scene].__id__ rescue nil
+        a = self.class.new.replace super
+        a.set_cf cfid
+        return a
     end
 
     def reject!(&block)
         cfid = self[@scene].__id__ rescue nil
-        a = super
+        a = super       # can return nil
         set_cf cfid
         return a
     end
@@ -1245,6 +1355,9 @@ public
             cfid = self[@scene].__id__ rescue nil
         end
         super
+        # set_cf will fail if the new list has fewer images
+        # than the scene number indicates.
+        @scene = self.length == 0 ? nil : 0
         set_cf cfid
         self
     end
@@ -1263,13 +1376,11 @@ public
         return a
     end
 
-    if Array.instance_methods(false).include? 'select' then
-        def select(*args,&block)
-            cfid = self[@scene].__id__ rescue nil
-            a = super
-            a.set_cf cfid
-            return a
-        end
+    def select(&block)
+        cfid = self[@scene].__id__ rescue nil
+        a = self.class.new.replace super
+        a.set_cf cfid
+        return a
     end
 
     def shift
@@ -1330,11 +1441,24 @@ public
             r = self[x] <=> other[x]
             return r unless r == 0
         end
+        if @scene.nil? && other.scene.nil?
+            return 0
+        elsif @scene.nil? && ! other.scene.nil?
+            raise TypeError, "cannot convert nil into #{other.scene.class}"
+        elsif ! @scene.nil? && other.scene.nil?
+            raise TypeError, "cannot convert nil into #{self.scene.class}"
+        end
         r = self.scene <=> other.scene
         return r unless r == 0
         return self.length <=> other.length
     end
 
+    def clone
+        ditto = dup
+        ditto.freeze if frozen?
+        return ditto
+    end
+
     # Make a deep copy
     def copy
         ditto = self.class.new
@@ -1355,11 +1479,26 @@ public
     # Set same delay for all images
     def delay=(d)
         if Integer(d) < 0
-            raise ArgumentError, "delay must be greater than 0"
+            raise ArgumentError, "delay must be greater than or equal to 0"
         end
         each { |f| f.delay = Integer(d) }
     end
 
+    def ticks_per_second=(t)
+        if Integer(t) < 0
+            raise ArgumentError, "ticks_per_second must be greater than or equal to 0"
+        end
+        each { |f| f.ticks_per_second = Integer(t) }
+    end
+
+    def dup
+        ditto = self.class.new
+        each {|img| ditto << img}
+        ditto.scene = @scene
+        ditto.taint if tainted?
+        return ditto
+    end
+
     def from_blob(*blobs, &block)
         if (blobs.length == 0)
             raise ArgumentError, "no blobs given"
@@ -1392,6 +1531,7 @@ public
 
     # Set the number of iterations of an animated GIF
     def iterations=(n)
+        n = Integer(n)
         if n < 0 || n > 65535
             raise ArgumentError, "iterations must be between 0 and 65535"
         end
@@ -1410,8 +1550,8 @@ public
             else
                 super
             end
-        rescue NameError
-          raise NameError, "undefined method `#{methID.id2name}' for #{self.class}"
+        rescue NoMethodError
+          raise NoMethodError, "undefined method `#{methID.id2name}' for #{self.class}"
         rescue Exception
             $@.delete_if { |s| /:in `send'$/.match(s) || /:in `method_missing'$/.match(s) }
             raise
